#include "header\GBA.h"
#include "header\BG.h"
#include "header\Timer.h"
#include "header\DMA.h"
#include "header\IRQ.h"
#include "header\sound.h"
#include "header\sprite.h"

#include "lib.h"
#include "iwram.h"
#include "irq.h"
#include "data.h"
#include "syscall.h"

#include "resource\BitMapFont.h"

//const int __gba_multiboot;  //GBANp

#define SPNO_BALL 0
#define SPNO_BAR1 1
#define SPNO_BAR2 5
#define SPNO_BLOCKGREEN 9
#define SPNO_BLOCKBLUE 13
#define SPNO_BLOCKRED 17

#define BLOCK_X_ARRAY 7
#define BLOCK_Y_ARRAY 3

#define DISPLAY_X_SIZE 240
#define DISPLAY_Y_SIZE 160
//---------------------------------------------------------------------------
typedef struct {
  u8 SPNo;  //XvCgmn
  u8 cx;
  u8 cy;
  s16 x;	//W
  s16 y;
  s16 dx;	//ω
  s16 dy;
  s16 xLeft; //ړEl
  s16 xRight;
  s16 yUp;
  s16 yDown;
} ST_BALL;

typedef struct {
  u8 SPNo1;  //XvCgmn
  u8 SPNo2;  //XvCgmn
  u8 cx;
  u8 cy;
  s16 x;
  s16 y;
  s16 dx;
  s16 xLeft; //ړEl
  s16 xRight;
} ST_BAR;

typedef struct {
  u8 SPNo;          //XvCgmn
  u8 stat;          //Xe[^X
  u8 x;             //\JnW
  u8 y;
  u8 cx;            //32 (32*7=224)
  u8 cy;            //32
} ST_BLOCK;

ST_BALL BALL;
ST_BAR BAR;
ST_BLOCK BLOCK[BLOCK_Y_ARRAY][BLOCK_X_ARRAY];
u8 BlockCount = BLOCK_Y_ARRAY * BLOCK_X_ARRAY; //28 (7*4=28) cubN

//---------------------------------------------------------------------------
void init();
void init_paint();
void paint();
void input();
void movehit();
int chkblock();
//---------------------------------------------------------------------------

int main(void)
{
  //u16 i;
  u8 BG1_Chr = 1;
  u8 BG1_Map = 0;
  REG_BG1CNT = ( BG_SIZEA_512_512 |
                 BG_COLOR_16 |
                 BG_CHARBASE(BG1_Chr) |
                 BG_MAPBASE(BG1_Map) |
                 0); 

  SpriteInitialize();
  SpriteSetPal16( (u16*)exttfs_GetDataOffset(_GAME_PAL_BIN_OFFSET) );
  SpriteSetChr(   (u16*)exttfs_GetDataOffset(_GAME_IMG_BIN_OFFSET),
                  _GAME_IMG_BIN_SIZE );

  init();
  init_paint();
  SetMode( MODE_1 | BG1_ENABLE | OBJ_ENABLE | OBJ_MAP_2D );

  // Push Start NWbg

  // C[v
  while(BlockCount){
    movehit();
    paint();
    input();

    WaitForVsync(); // ҂
  }

  dprintf("GAME CLEAR\n");
  for(;;){}
  return 0;
}
//---------------------------------------------------------------------------
void movehit(){
  //xW
  BALL.x += BALL.dx;
  if(BALL.x < BALL.xLeft || BALL.x > BALL.xRight)
    BALL.dx *= -1;

  //ubNx
  if( chkblock() ){
    BALL.dx *= -1;
  }

  //yW
  BALL.y += BALL.dy;
  if(BALL.y < BALL.yUp || BALL.y > BALL.yDown)
    BALL.dy *= -1;

  //ubNy
  if( chkblock() ){
    BALL.dy *= -1;
  }

  //Pbg
  if(BALL.y+BALL.cy > BAR.y && BALL.y < BAR.y+BAR.cy){
    if(BALL.x+BALL.cx > BAR.x && BALL.x < BAR.x+BAR.cx){
      BALL.dy *= -1;
    }
  }

}
//---------------------------------------------------------------------------
int chkblock(){
  int i,j;
  for(i=0; i<BLOCK_Y_ARRAY; i++){
    for(j=0; j<BLOCK_X_ARRAY; j++){
      if(BLOCK[i][j].stat){
        if(BALL.y+BALL.cy > BLOCK[i][j].y && BALL.y < BLOCK[i][j].y+BLOCK[i][j].cy){
          if(BALL.x+BALL.cx > BLOCK[i][j].x && BALL.x < BLOCK[i][j].x+BLOCK[i][j].cx){
            SpriteMove(BLOCK[i][j].SPNo, 240, 160); //ʊO
            BLOCK[i][j].stat = 0;
            BlockCount--;

            return 1;
          }
        }
      }
    }
  }
  return 0;
}
//---------------------------------------------------------------------------
void init_paint(){
  //block
  int i,j;
  for(i=0; i<BLOCK_Y_ARRAY; i++){
    for(j=0; j<BLOCK_X_ARRAY; j++){
      if(BLOCK[i][j].stat){
        SpriteMove(BLOCK[i][j].SPNo, BLOCK[i][j].x, BLOCK[i][j].y);
      }
    }
  }
}
//---------------------------------------------------------------------------
void paint(){
  //ball
  SpriteMove(BALL.SPNo, BALL.x, BALL.y);

  //bar
  SpriteMove(BAR.SPNo1, BAR.x,    BAR.y);
  SpriteMove(BAR.SPNo2, BAR.x+32, BAR.y);
}
//---------------------------------------------------------------------------
void input(){
  u32 key = *KEYS;

  //LEFT key
  if(!(key & KEY_LEFT)){
    BAR.x-=BAR.dx;
    if(BAR.x < BAR.xLeft)
       BAR.x = BAR.xLeft;
  }
  //RIGHT key
  if(!(key & KEY_RIGHT)){
    BAR.x+=BAR.dx;
    if(BAR.x > BAR.xRight)
       BAR.x = BAR.xRight;
  }
}
//---------------------------------------------------------------------------
void init(){

  //Ball
  BALL.SPNo   = 0;
  BALL.cx     = 8;
  BALL.cy     = 8;
  BALL.x      = 120-(BALL.cx/2);   //W
  BALL.y      = 130;
  BALL.dx     = 3;	 //ω
  BALL.dy     = -2;
  BALL.xLeft  = 8; //ړEl
  BALL.xRight = (240-8)-BALL.cx;
  BALL.yUp    = 2;
  BALL.yDown  = 160-BALL.cy;
  SpriteSetSize(BALL.SPNo, SP_SIZE_8 ,SP_SQUARE,SP_COLOR_16);
  SpriteChange(BALL.SPNo, SPNO_BALL);
  SpriteSetPalNo(BALL.SPNo, 0);

  //bar
  BAR.SPNo1  = 1;
  BAR.SPNo2  = 2;
  BAR.cx     = 32+32;
  BAR.cy     = 0;    //8邯ǔł0
  BAR.x      = 120-(BAR.cx/2);
  BAR.y      = 140;
  BAR.dx     = 3;	 //ω
  BAR.xLeft  = 2;    //ړEl
  BAR.xRight = 238 - BAR.cx;
  SpriteSetSize(BAR.SPNo1, SP_SIZE_16,SP_WIDE  ,SP_COLOR_16);
  SpriteSetSize(BAR.SPNo2, SP_SIZE_16,SP_WIDE  ,SP_COLOR_16);
  SpriteChange(BAR.SPNo1, SPNO_BAR1);
  SpriteChange(BAR.SPNo2, SPNO_BAR2);
  SpriteSetPalNo(BAR.SPNo1,0);
  SpriteSetPalNo(BAR.SPNo2,0);

  //block
  int i;
  for(i=0; i<BLOCK_X_ARRAY; i++){
    //green
    SpriteSetSize( 10+i, SP_SIZE_32,SP_WIDE,SP_COLOR_16);
    SpriteChange(  10+i, SPNO_BLOCKGREEN);
    SpriteSetPalNo(10+i, 0);

    //blue
    SpriteSetSize( 20+i, SP_SIZE_32,SP_WIDE,SP_COLOR_16);
    SpriteChange(  20+i, SPNO_BLOCKBLUE);
    SpriteSetPalNo(20+i, 0);

    //red
    SpriteSetSize( 30+i, SP_SIZE_32,SP_WIDE,SP_COLOR_16);
    SpriteChange(  30+i, SPNO_BLOCKRED);
    SpriteSetPalNo(30+i, 0);
  }

  //green
  for(i=0; i<BLOCK_X_ARRAY; i++){
      BLOCK[0][i].SPNo = 10+i;
      BLOCK[0][i].stat = 1;
      BLOCK[0][i].cx   = 32;
      BLOCK[0][i].cy   = 16;
      BLOCK[0][i].x    = 8+i * BLOCK[0][i].cx;
      BLOCK[0][i].y    = 2;
  }
  //blue
    for(i=0; i<BLOCK_X_ARRAY; i++){
      BLOCK[1][i].SPNo = 20+i;
      BLOCK[1][i].stat = 1;
      BLOCK[1][i].cx   = 32;
      BLOCK[1][i].cy   = 16;
      BLOCK[1][i].x    = 8+i * BLOCK[1][i].cx;
      BLOCK[1][i].y    = 2+16;
  }
  //red
    for(i=0; i<BLOCK_X_ARRAY; i++){
      BLOCK[2][i].SPNo = 30+i;
      BLOCK[2][i].stat = 1;
      BLOCK[2][i].cx   = 32;
      BLOCK[2][i].cy   = 16;
      BLOCK[2][i].x    = 8+i * BLOCK[2][i].cx;
      BLOCK[2][i].y    = 2+16+16;
  }
}
